"""
Setup configuration for kaka-auto.

This file is used for editable installs (pip install -e .)
and is compatible with setuptools using pyproject.toml.
"""

from setuptools import setup

# All configuration is in pyproject.toml
setup()
